@@ -6,7 +6,7 @@ use 5.010;
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Carp qw(confess cluck);
 use DateTime;
@@ -15,11 +15,14 @@ use List::MoreUtils qw(firstval none uniq);
 use LWP::UserAgent;
 use Text::CSV;
 use Travel::Status::DE::URA::Result;
+use Travel::Status::DE::URA::Stop;
 
 sub new {
 	my ( $class, %opt ) = @_;
 
-	my $ua = LWP::UserAgent->new(%opt);
+	my %lwp_options = %{ $opt{lwp_options} // { timeout => 10 } };
+
+	my $ua = LWP::UserAgent->new(%lwp_options);
 	my $response;
 
 	if ( not( $opt{ura_base} and $opt{ura_version} ) ) {
@@ -29,13 +32,14 @@ sub new {
 	my $self = {
 		datetime => $opt{datetime}
 		  // DateTime->now( time_zone => 'Europe/Berlin' ),
-		ura_base    => $opt{ura_base},
-		ura_version => $opt{ura_version},
-		full_routes => $opt{full_routes} // 0,
-		hide_past   => $opt{hide_past} // 1,
-		stop        => $opt{stop},
-		via         => $opt{via},
-		post        => {
+		developer_mode => $opt{developer_mode},
+		ura_base       => $opt{ura_base},
+		ura_version    => $opt{ura_version},
+		full_routes    => $opt{calculate_routes} // 0,
+		hide_past      => $opt{hide_past} // 1,
+		stop           => $opt{stop},
+		via            => $opt{via},
+		post           => {
 			ReturnList =>
 			  'lineid,linename,directionid,destinationtext,vehicleid,'
 			  . 'tripid,estimatedtime,stopid,stoppointname'
@@ -63,6 +67,10 @@ sub new {
 
 	$self->{raw_str} = $response->decoded_content;
 
+	if ( $self->{developer_mode} ) {
+		say $self->{raw_str};
+	}
+
 	# Fix encoding in case we're running through test files
 	if ( substr( $self->{ura_instant_url}, 0, 5 ) eq 'file:' ) {
 		$self->{raw_str} = encode( 'UTF-8', $self->{raw_str} );
@@ -120,16 +128,16 @@ sub results {
 	my ( $self, %opt ) = @_;
 	my @results;
 
-	my $full_routes = $opt{full_routes} // $self->{full_routes} // 0;
-	my $hide_past   = $opt{hide_past}   // $self->{hide_past}   // 1;
-	my $stop        = $opt{stop}        // $self->{stop};
-	my $via         = $opt{via}         // $self->{via};
+	my $full_routes = $opt{calculate_routes} // $self->{full_routes} // 0;
+	my $hide_past   = $opt{hide_past}        // $self->{hide_past}   // 1;
+	my $stop        = $opt{stop}             // $self->{stop};
+	my $via         = $opt{via}              // $self->{via};
 
 	my $dt_now = $self->{datetime};
 	my $ts_now = $dt_now->epoch;
 
 	if ($via) {
-		$full_routes ||= 'after';
+		$full_routes = 1;
 	}
 
 	for my $dep ( @{ $self->{raw_list} } ) {
@@ -138,7 +146,7 @@ sub results {
 			$u1, $stopname, $stopid,    $lineid, $linename,
 			$u2, $dest,     $vehicleid, $tripid, $timestamp
 		) = @{$dep};
-		my @route;
+		my ( @route_pre, @route_post );
 
 		if ( $stop and not( $stopname eq $stop ) ) {
 			next;
@@ -162,52 +170,61 @@ sub results {
 		my $ts_dep = $dt_dep->epoch;
 
 		if ($full_routes) {
-			@route = map { [ $_->[9] / 1000, $_->[1] ] }
+			my @route = map { [ $_->[9] / 1000, $_->[1] ] }
 			  grep { $_->[8] == $tripid } @{ $self->{raw_list} };
 
-			if ( $full_routes eq 'before' ) {
-				@route = grep { $_->[0] < $ts_dep } @route;
-			}
-			elsif ( $full_routes eq 'after' ) {
-				@route = grep { $_->[0] > $ts_dep } @route;
-			}
+			@route_pre  = grep { $_->[0] < $ts_dep } @route;
+			@route_post = grep { $_->[0] > $ts_dep } @route;
 
 			if ( $via
-				and none { $_->[1] eq $via } @route )
+				and none { $_->[1] eq $via } @route_post )
 			{
 				next;
 			}
 
 			if ($hide_past) {
-				@route = grep { $_->[0] >= $ts_now } @route;
+				@route_pre = grep { $_->[0] >= $ts_now } @route_pre;
 			}
 
-			@route = map { $_->[0] }
+			@route_pre = map { $_->[0] }
 			  sort { $a->[1] <=> $b->[1] }
-			  map { [ $_, $_->[0] ] } @route;
+			  map { [ $_, $_->[0] ] } @route_pre;
+			@route_post = map { $_->[0] }
+			  sort { $a->[1] <=> $b->[1] }
+			  map { [ $_, $_->[0] ] } @route_post;
 
-			@route = map {
-				[
-					DateTime->from_epoch(
+			@route_pre = map {
+				Travel::Status::DE::URA::Stop->new(
+					datetime => DateTime->from_epoch(
 						epoch     => $_->[0],
 						time_zone => 'Europe/Berlin'
 					),
-					decode( 'UTF-8', $_->[1] )
-				]
-			} @route;
+					name => decode( 'UTF-8', $_->[1] )
+				  )
+			} @route_pre;
+			@route_post = map {
+				Travel::Status::DE::URA::Stop->new(
+					datetime => DateTime->from_epoch(
+						epoch     => $_->[0],
+						time_zone => 'Europe/Berlin'
+					),
+					name => decode( 'UTF-8', $_->[1] )
+				  )
+			} @route_post;
 		}
 
 		push(
 			@results,
 			Travel::Status::DE::URA::Result->new(
-				datetime        => $dt_dep,
-				dt_now          => $dt_now,
-				line            => $linename,
-				line_id         => $lineid,
-				destination     => decode( 'UTF-8', $dest ),
-				route_timetable => [@route],
-				stop            => $stopname,
-				stop_id         => $stopid,
+				datetime    => $dt_dep,
+				dt_now      => $dt_now,
+				line        => $linename,
+				line_id     => $lineid,
+				destination => decode( 'UTF-8', $dest ),
+				route_pre   => [@route_pre],
+				route_post  => [@route_post],
+				stop        => $stopname,
+				stop_id     => $stopid,
 			)
 		);
 	}
@@ -249,13 +266,14 @@ API" data providers (e.g. ASEAG)
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 DESCRIPTION
 
-Travel::Status::DE::URA is an unofficial interface URA-based realtime departure
-monitors (as used e.g. by the ASEAG).  It reports all upcoming departures at a
-given place in real-time.  Schedule information is not included.
+Travel::Status::DE::URA is an unofficial interface to URA-based realtime
+departure monitors (as used e.g. by the ASEAG).  It reports all upcoming
+departures at a given place in real-time.  Schedule information is not
+included.
 
 =head1 METHODS
 
@@ -281,13 +299,24 @@ The version, may be any string.
 =back
 
 The request URL is I<ura_base>/instant_VI<version>, so for
-C<< http://ivu.aseag.de/interfaces/ura >>, C<< 1 >> this module will point
+C<< http://ivu.aseag.de/interfaces/ura >>, C<< 1 >> this module will send
 requests to C<< http://ivu.aseag.de/interfaces/ura/instant_V1 >>.
 
+The following parameter is optional:
+
+=over
+
+=item B<lwp_options> => I<\%hashref>
+
+Passed on to C<< LWP::UserAgent->new >>. Defaults to C<< { timeout => 10 } >>,
+you can use an empty hashref to override it.
+
+=back
+
 Additionally, all options supported by C<< $status->results >> may be specified
-here, causing them to be used as defaults. Note that while they may be
-overridden later, they may limit the set of available departures requested from
-the server.
+here, causing them to be used as defaults. Note that while they can be
+overridden later, they may limit the set of departures requested from the
+server.
 
 =item $status->errstr
 
@@ -310,14 +339,11 @@ Accepted parameters (all are optional):
 
 =over
 
-=item B<full_routes> => B<before>|B<after>|I<bool> (default 0)
+=item B<calculate_routes> => I<bool> (default 0)
 
-When set to a true value: Compute B<route_timetable> fields in all
-Travel::Status::DE::URA::Result(3pm) objects, otherwise they will not be
-set.
-
-B<before> / B<after> limits the timetable to stops before / after the stop
-I<name> (if set).
+When set to a true value: Compute routes for all results, enabling use of
+their B<route_> accessors. Otherwise, those will just return nothing
+(undef / empty list, depending on context).
 
 =item B<hide_past> => I<bool> (default 1)
 
@@ -329,11 +355,8 @@ Only return departures at stop I<name>.
 
 =item B<via> => I<vianame>
 
-Only return departures containing I<vianame> in their route. If B<stop> is set,
-I<vianame> must be in the route after the stop I<name>. If, in addition to
-that, B<full_routes> is set to B<before>, I<vianame> must be in the route
-before the stop I<name>. Implies C<< full_routes> => 'after' >> unless
-B<full_routes> is explicitly set to B<before> / B<after> / 1.
+Only return departures containing I<vianame> in their route after their
+corresponding stop. Implies B<calculate_routes>=1.
 
 =back
 
@@ -355,6 +378,8 @@ None.
 
 =item * LWP::UserAgent(3pm)
 
+=item * Text::CSV(3pm)
+
 =back
 
 =head1 BUGS AND LIMITATIONS
@@ -367,7 +392,7 @@ Travel::Status::DE::URA::Result(3pm).
 
 =head1 AUTHOR
 
-Copyright (C) 2013 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2013-2015 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
