@@ -8,7 +8,7 @@ use parent 'Class::Accessor';
 
 use DateTime::Format::Duration;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 Travel::Status::DE::URA::Result->mk_ro_accessors(
 	qw(datetime destination line line_id stop stop_id));
@@ -56,10 +56,65 @@ sub type {
 	return 'Bus';
 }
 
-sub route_timetable {
+sub route_interesting {
+	my ( $self, $max_parts ) = @_;
+
+	my @via = $self->route_post;
+	my ( @via_main, @via_show, $last_stop );
+	$max_parts //= 3;
+
+	for my $stop (@via) {
+		if (
+			$stop->name =~ m{ bf | hbf | Flughafen | bahnhof
+				| Krankenhaus | Klinik | bushof | busstation }iox
+		  )
+		{
+			push( @via_main, $stop );
+		}
+	}
+	$last_stop = pop(@via);
+
+	if ( @via_main and $via_main[-1] == $last_stop ) {
+		pop(@via_main);
+	}
+	if ( @via and $via[-1] == $last_stop ) {
+		pop(@via);
+	}
+
+	if ( @via_main and @via and $via[0] == $via_main[0] ) {
+		shift(@via_main);
+	}
+
+	if ( @via < $max_parts ) {
+		@via_show = @via;
+	}
+	else {
+		if ( @via_main >= $max_parts ) {
+			@via_show = ( $via[0] );
+		}
+		else {
+			@via_show = splice( @via, 0, $max_parts - @via_main );
+		}
+
+		while ( @via_show < $max_parts and @via_main ) {
+			my $stop = shift(@via_main);
+			push( @via_show, $stop );
+		}
+	}
+
+	return @via_show;
+}
+
+sub route_pre {
 	my ($self) = @_;
 
-	return @{ $self->{route_timetable} };
+	return @{ $self->{route_pre} };
+}
+
+sub route_post {
+	my ($self) = @_;
+
+	return @{ $self->{route_post} };
 }
 
 sub TO_JSON {
@@ -89,7 +144,7 @@ departure received by Travel::Status::DE::URA
 
 =head1 VERSION
 
-version 0.04
+version 0.05
 
 =head1 DESCRIPTION
 
@@ -133,18 +188,32 @@ The name of the line.
 
 The number of the line.
 
-=item $departure->route_timetable
+=item $departure->route_interesting(I<num_stops>)
 
 If the B<results> method of Travel::Status::DE::URA(3pm) was called with
-B<full_routes> => true:
-Returns a list of arrayrefs describing the entire route. I.e.
-C<< ([$time1, $stop1], [$time2, $stop2], ...) >>.
-The times are DateTime::Duration(3pm) objects, the stops are only names,
-not IDs (subject to change).  Returns an empty list otherwise.
+B<calculate_routes> => true: Returns a list of up to I<num_stops> (defaults to
+3) stops considered interesting (usually of major importance in the transit
+area). Each stop is a Travel::Status::DE::URA::Stop(3pm) object.  Note that the
+importance is determined heuristically based on the stop name, so it is not
+always accurate.
+
+Returns an empty list if B<calculate_routes> was false.
+
+=item $departure->route_pre
+
+If the B<results> method of Travel::Status::DE::URA(3pm) was called with
+B<calculate_routes> => true:
+Returns a list containing all stops after the requested one.
+Each stop is a Travel::Status::DE::URA::Stop(3pm) object.
+Returns an empty list otherwise.
+
+=item $departure->route_post
+
+Same as B<route_pre>, but contains the stops before the requested one.
 
 =item $departure->stop
 
-The stop belonging to this departure.
+The stop (name, not object) belonging to this departure.
 
 =item $departure->stop_id
 
@@ -194,11 +263,11 @@ Unknown.
 
 =head1 SEE ALSO
 
-Travel::Status::DE::URA(3pm).
+Travel::Status::DE::URA(3pm), Travel::Status::DE::URA::Stop(3pm).
 
 =head1 AUTHOR
 
-Copyright (C) 2013 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2013-2015 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
